﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;

namespace FVC10_GrabSample
{
    public partial class Form_FVC10_GRABSAMPLE : Form
    {
        //画像メモリ
        private List<FVIL.Data.CFviImage> m_Images = new List<FVIL.Data.CFviImage>();
        private FVIL.Data.CFviImage m_Image = new FVIL.Data.CFviImage();

        //FCI関連
        private FVIL.FCI.CFviFCI m_fci = new FVIL.FCI.CFviFCI();
        private FVIL.FCI.CFviFCIGrabberFVC10 m_grabber = new FVIL.FCI.CFviFCIGrabberFVC10();

        //時間計測
        private FVIL.CFviTimeCounter m_timer = new FVIL.CFviTimeCounter();
        
        // thread
        // スレッド状態
        public enum ThreadStatus
        {
            BUSY = 1,
            DEAD = 2,
        }
        public enum ThreadOrder
        {
            STOP = 0,
            RUN = 1,
        }


        Thread m_threadCapture = null;
        ThreadOrder m_ThreadOrder  = ThreadOrder.STOP;
	    ThreadStatus m_ThreadStatus = ThreadStatus.DEAD;


        public Form_FVC10_GRABSAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォーム表示開始
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 使用許諾を確認します。
        /// </remarks>
        private void Form1_Shown(object sender, EventArgs e)
        {
            dialogLicence licence_dialog = new dialogLicence();
            if ((licence_dialog.ShowDialog(this) != DialogResult.OK))
            {
                this.Close();
                //使用許諾が得られない場合はサンプルプログラムの動作を終了
                return;
            }

            //カメラファイル選択
            fnCameraSetting();

        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void Form1_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();


            //計測時間テキストボックス初期化
            TextBoxTime.Text = "0.000";
            //フレーム数テキストボックス初期化
            TextBoxFrame.Text = "0";

            //カメラタイプコンボボックス初期化
            comboBoxScanType.Items.Clear();
            comboBoxScanType.Items.Add("Areascan");
            comboBoxScanType.Items.Add("Linescan");
            comboBoxScanType.SelectedIndex = 0;

            //トリガモードコンボボックス初期化
            ComboBoxTrigger.Items.Clear();
            ComboBoxTrigger.Items.Add("ノーマル");
            ComboBoxTrigger.Items.Add("内部トリガ");
            ComboBoxTrigger.Items.Add("外部トリガ");
            ComboBoxTrigger.SelectedIndex = 0;
            
            //エンコーダモード コンボボックス初期化
            comboBoxEncoderMode.Items.Clear();
            comboBoxEncoderMode.Items.Add("エンコーダを使用しない");
            comboBoxEncoderMode.Items.Add("相対位置エンコーダ x1 CW");
            comboBoxEncoderMode.Items.Add("相対位置エンコーダ x2 CW");
            comboBoxEncoderMode.Items.Add("相対位置エンコーダ x4 CW");
            comboBoxEncoderMode.Items.Add("相対位置エンコーダ x4 CCW");
            comboBoxEncoderMode.Items.Add("絶対位置エンコーダ x4 CW");
            comboBoxEncoderMode.Items.Add("絶対位置エンコーダ x4 CCW");
            comboBoxEncoderMode.SelectedIndex = 0;

            //エンコーダリセット コンボボックス初期化
            comboBoxResetsource.Items.Clear();
            comboBoxResetsource.Items.Add("初期化しない");
            comboBoxResetsource.Items.Add("1フレーム完了後リセット");
            comboBoxResetsource.Items.Add("全取り込み完了後リセット");
            comboBoxResetsource.SelectedIndex = 0;

            //エンコーダ入力チャネル コンボボックス初期化
            comboBoxSignalsource.Items.Clear();
            comboBoxSignalsource.Items.Add("エンコーダ入力チャネル #0");
            comboBoxSignalsource.Items.Add("エンコーダ入力チャネル #1");
            comboBoxSignalsource.SelectedIndex = 0;

            //エンコーダ起動トリガ コンボボックス初期化
            comboBoxTriggersource.Items.Clear();
            comboBoxTriggersource.Items.Add("未使用");
            comboBoxTriggersource.Items.Add("外部トリガ");
            comboBoxTriggersource.Items.Add("内部トリガ(ソフトトリガ)");
            comboBoxTriggersource.Items.Add("Z相");
            comboBoxTriggersource.SelectedIndex = 0;

            //ノーマルモード時は、露光時間、トリガ発行間隔コントロールを無効にする
            NumericUpDownExposure.Enabled = false;
            NumericUpDownCycleTime.Enabled = false;
        }


        /// <summary>
        /// カメラ設定ファイル選択関数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// iniファイルの選択をします。
        /// </remarks>
        private void fnCameraSetting()
        {
            Cursor preCursor = Cursor.Current;
            //xmlファイル設定画面
            Form_BROWSEIN formbroseini = new Form_BROWSEIN();

            //xmlファイル名（フルパス）
            string strCameraFilePath = null;
            int intSendCameraCommand = -1;
            int initialTimeout = 15000;

            //xmlファイルの選択
            if (formbroseini.ShowDialog(this) == DialogResult.OK)
            {
                strCameraFilePath = formbroseini.strCameraFilePath;
                intSendCameraCommand = formbroseini.intSendCameraCommand;
                initialTimeout = formbroseini.initialTimeout;
            }

            if (strCameraFilePath == null)
            {
                //xmlファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }


            if (strCameraFilePath.Length > 0)
            {
                try
                {
                    //デバイスがオープンされていた場合はクローズする
                    if (m_grabber != null)
                    {
                        if (m_grabber.IsAnnounced)
                        {
                            m_grabber.RevokeBuffer();
                        }
                        if (m_grabber.IsOpened)
                        {
                            m_grabber.Close();
                        }
                    }

                    Cursor.Current = Cursors.WaitCursor;
                    m_fci.Initialize();  
                    m_fci.Discover(initialTimeout);
                    m_grabber.Open(m_fci);
                    m_grabber.Setup(strCameraFilePath, intSendCameraCommand);

                    // パラメータ取得
                    ComboBoxTrigger.SelectedIndex = (int)m_grabber.Trigger;
                    NumericUpDownExposure.Value = m_grabber.ExposureTime;
                    NumericUpDownCycleTime.Value = (int)m_grabber.TriggerCycleTime;

                    //カメラタイプ取得
                    string str = (string)m_grabber.GetParam("ScanType");
                    if (str == "Areascan")
                    {
                        comboBoxScanType.SelectedIndex = 0;
                    }
                    else
                    {
                        comboBoxScanType.SelectedIndex = 1;
                    }

                    //エンコーダパラメータ取得
                    comboBoxEncoderMode.SelectedIndex = m_grabber.EncoderMode;
                    numericUpDownDelay.Value = m_grabber.EncoderConfig.Delay;
                    numericUpDownPeriod.Value = m_grabber.EncoderConfig.Period;
                    comboBoxResetsource.SelectedIndex = (int)m_grabber.EncoderConfig.Resetsource;
                    comboBoxSignalsource.SelectedIndex = (int)m_grabber.EncoderConfig.Signalsource;
                    comboBoxTriggersource.SelectedIndex = (int)m_grabber.EncoderConfig.Triggersource;

                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                finally
                {
                    Cursor.Current = preCursor;
                }
            }
            else
            {
                //xmlファイルが未選択の場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }

        }

        /// <summary>
        /// Grab Oneボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込み。
        /// </remarks>
        private void ButtonGrabOne_Click(object sender, EventArgs e)
        {

            double msec = 0;

            try
            {
                //フレーム数テキストボックス初期化
                TextBoxFrame.Text = "0";

                //取込画像を正規化
                m_grabber.ChangeImageSize(m_Image);

                //画像メモリ登録＆キュー
                m_grabber.AnnounceAndQueueBuffer(m_Image);

                // Start
                m_grabber.CameraStart();
                m_grabber.GrabStart(1);

                //時間測定開始
                m_timer.Start();
                //内部トリガモード時はソフトウェアトリガを発行する
                if (m_grabber.Trigger == 1)
                {
                    m_grabber.ForcedTrigger();
                }

                //バッファ取得
                FVIL.FCI.CFviBufferInfo info = m_grabber.GetBuffer();

                //時間測定終了
                msec = m_timer.Stop();

                //取込枚数取得
                TextBoxTime.Text = msec.ToString("0.000");
                ulong framecnt = (ulong)m_grabber.GetDataStreamInfo(FVIL.FCI.StreamInfoCmd.NumDelivered);
                TextBoxFrame.Text = framecnt.ToString();

                //描画imageのセット
                cFviImageView1.Image = m_Image;
                //再描画
                cFviImageView1.Refresh();
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                m_grabber.GrabStop();
                m_grabber.CameraStop();
                m_grabber.RevokeBuffer();
            }
        }

        /// <summary>
        /// Grab Continueボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み開始・停止。
        /// </remarks>
        private void ButtonGrabContinue_Click(object sender, EventArgs e)
        {

            if (m_ThreadStatus != ThreadStatus.DEAD)
                fnCaptureStopThread();
            else
            {
                try
                {

                    //前回の画像メモリ格納領域クリア
                    for (int imageindex = 0; imageindex < m_Images.Count; imageindex++)
                    {
                        m_Images[imageindex].Dispose();
                    }

                    //画像メモリ正規化
                    //取込みを連続で実行すると間に合わなくなる可能性があるので、サンプルでは画像メモリを5枚使用する
                    m_Images.Clear();
                    for (int imageindex = 0; imageindex < 5; imageindex++)
                    {
                        m_Images.Add(new FVIL.Data.CFviImage());
                        m_grabber.ChangeImageSize(m_Images[imageindex]);
                    }

                    // パラメータ操作禁止
                    ButtonGrabContinue.Text = "Stop";
                    ButtonGrabOne.Enabled = false;
                    TextBoxTime.Text = "0.000";
                    ComboBoxTrigger.Enabled = false;
                    GroupBoxEncoder.Enabled = false;
                    GroupBoxScanType.Enabled = false;
                    
                    // Thread開始
                    m_ThreadOrder = ThreadOrder.RUN;
                    m_threadCapture = new Thread(new ThreadStart(fnCaptureThread));
                    m_threadCapture.Start();

                }
                catch (FVIL.CFviException ex)
                {
                    //エラー処理
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (System.Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            Application.DoEvents();

        }

        delegate void UpdateCaptureImageDelegate(double _msec, uint _imageindex, ulong frameid);
        delegate void UpdateStopThread();

        /// <summary>
        /// カメラ取り込みスレッド
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 連続カメラ取り込み。
        /// </remarks>
        private void fnCaptureThread()
        {
            UpdateCaptureImageDelegate updateinfo = new UpdateCaptureImageDelegate(this.fnUpdateCaptureInfo);
            UpdateStopThread updatestop = new UpdateStopThread(this.fnUpdateStopThread);
            double msec = 0;
            ulong frameid = 0;
            m_ThreadStatus = ThreadStatus.BUSY;

            try
            {
                //バッファ登録
                m_grabber.AnnounceBuffer(m_Images);

                //入出力バッファリセット
                m_grabber.ResetQueue();

                //取込開始
                m_grabber.CameraStart();
                //無限取込
                m_grabber.GrabStart(0);

                m_timer.Start();
                //サンプルプログラムでは、内部トリガモード時は、コーディングにてソフトウェアトリガの最初の1回目を発行し、
                //以降は、トリガリトライモードの設定「連続出力する」にて、ソフトウェアトリガを発行して、取り込みしています
                if (m_grabber.Trigger == 1)
                {
                    m_grabber.ForcedTrigger();
                }
                while (true)
                {
                    if (m_ThreadOrder == ThreadOrder.STOP)
                    {
                        break;
                    }

                    if (!m_grabber.IsGrabbing)
                    {
                        break;
                    }

                    // バッファ取得
                    FVIL.FCI.CFviBufferInfo info = m_grabber.GetBuffer(5000);
                    msec = m_timer.GetTime();

                    frameid = (ulong)m_grabber.GetFrameID(info.ImageIndex);

                    // キューバッファ
                    m_grabber.QueueBuffer(info.ImageIndex);

                    // 描画
                    BeginInvoke(updateinfo, new object[] { msec, info.ImageIndex, frameid });

                }
            }
            catch (FVIL.CFviException ex)
            {
                // buffer.waitがキャンセルされたとき、Abortを返す
                if (ex.ErrorCode != FVIL.FCI.ErrorCode.ABORT)
                {
                    MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception)
            {
            }
            finally
            {
                //取り込み停止
                m_grabber.GrabStop();
                m_grabber.CameraStop();

                // バッファ登録解除
                m_grabber.RevokeBuffer();

            }
            m_ThreadStatus = ThreadStatus.DEAD;
            BeginInvoke(updatestop, new object[] { });
        }

        /// <summary>
        /// 取り込み情報更新
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 取り込み情報を更新します。
        /// </remarks>
        void fnUpdateCaptureInfo(double _msec, uint _imageindex, ulong frameid)
        {
            TextBoxTime.Text = _msec.ToString("0.000");
            ulong framecnt = (ulong)m_grabber.GetDataStreamInfo(FVIL.FCI.StreamInfoCmd.NumDelivered);
            TextBoxFrame.Text = framecnt.ToString();
            cFviImageView1.Image = m_Images[(int)_imageindex];
            cFviImageView1.Refresh();
        }

        /// <summary>
        /// 取り込み停止スレッド
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ取り込みを停止します。
        /// </remarks>
        private void fnCaptureStopThread()
        {
            m_ThreadOrder = ThreadOrder.STOP;
            m_grabber.CancelGetBuffer();

            if (m_ThreadStatus != ThreadStatus.DEAD)
            {
                m_threadCapture.Join();
            }
         }

        /// <summary>
        /// 更新停止スレッド
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// スレッドを停止した時の処理。
        /// </remarks>
        private void fnUpdateStopThread()
        {

            ButtonGrabContinue.Text = "Grab Continue";
            ButtonGrabOne.Enabled = true;
            ComboBoxTrigger.Enabled = true;
            GroupBoxEncoder.Enabled = true;
            GroupBoxScanType.Enabled = true;
        }

        /// <summary>
        /// Trigger Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ノーマルモード・内部トリガモード・外部トリガモードの選択。
        /// </remarks>
        private void ComboBoxTrigger_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    m_grabber.Trigger = (uint)ComboBoxTrigger.SelectedIndex;
                    switch (ComboBoxTrigger.SelectedIndex)
                    {
                        //ノーマルモード
                        case 0:
                            //トリガリトライモード[0:連続出力しない]
                            m_grabber.TriggerRetryMode = 0;
                            NumericUpDownExposure.Enabled = false;
                            NumericUpDownCycleTime.Enabled = false;
                            break;
                        //内部トリガモード
                        case 1:
                            //サンプルプログラムでは、内部トリガモード時は、トリガリトライモードの設定で、
                            //ソフトウェアトリガを連続出力して、取り込みを行います
                            //トリガリトライモード[1:連続出力する]
                            m_grabber.TriggerRetryMode = 1;
                            NumericUpDownExposure.Enabled = true;
                            NumericUpDownCycleTime.Enabled = true;
                            break;
                        //外部トリガモード
                        case 2:
                            //トリガリトライモード[0:連続出力しない]
                            m_grabber.TriggerRetryMode = 0;
                            NumericUpDownExposure.Enabled = true;
                            NumericUpDownCycleTime.Enabled = true;
                            break;
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Video Exposure値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラ露光時間の設定。
        /// </remarks>
        private void NumericUpDownExposure_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    if (NumericUpDownExposure.Value < m_grabber.TriggerCycleTime)
                    {
                        m_grabber.ExposureTime = (uint)NumericUpDownExposure.Value;
                    }
                    else
                    {
                        //エラー処理
                        //トリガサイクルタイムの特性上、露光時間の設定はトリガサイクルタイム設定よりも小さくなければなりません。
                        //ExposureTime + 1Frame転送時間 ＜ TriggerCycleTime の条件を守って設定してください。
                        MessageBox.Show("カメラ露光時間はトリガ再発行間隔よりも小さくしてください。", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// TriggerCycleTime値
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// トリガリトライ期間の設定。
        /// </remarks>
        private void NumericUpDownCycleTime_ValueChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    if (m_grabber.ExposureTime < NumericUpDownCycleTime.Value)
                    {
                        m_grabber.TriggerCycleTime = (uint)NumericUpDownCycleTime.Value;
                    }
                    else
                    {
                        //エラー処理
                        //トリガサイクルタイムの特性上、露光時間の設定はトリガサイクルタイム設定よりも小さくなければなりません。
                        //ExposureTime + 1Frame転送時間 ＜ TriggerCycleTime の条件を守って設定してください。
                        MessageBox.Show("トリガ再発行間隔はカメラ露光時間よりも大きくしてください。", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// x1ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 等倍で表示します。
        /// </remarks>
        private void ButtonMag1_Click(object sender, EventArgs e)
        {
            //画像の表示倍率の取得または設定を行います。1.0と指定すると等倍で表示します。
            //倍率値は、0.5とした場合は、幅・高さ共に1/2となり、面積が1/4になります。
            //3.0とした場合は、幅・高さ共に3倍となり、面積が9倍になります。

            //表示倍率の設定
            cFviImageView1.Display.Magnification = 1.0;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Fitボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します。
        /// </remarks>
        private void ButtonFit_Click(object sender, EventArgs e)
        {
            //画像全体がビューのクライアント領域に表示できるように表示倍率を縮小(または拡大)します
            //表示倍率を調整
            cFviImageView1.FitImageSize();
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ＋ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率に0.5加えた倍率で表示します。
        /// </remarks>
        private void ButtonPlus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            //表示倍率の設定
            Value = Value + 0.5;
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// －ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 現在の倍率から0.5引いた倍率で表示します。ただし、倍率が０以下になる場合は倍率0.1で表示します。
        /// </remarks>
        private void ButtonMinus_Click(object sender, EventArgs e)
        {
            double Value = 0;
            //現倍率の取得
            Value = cFviImageView1.Display.Magnification;
            Value = Value - 0.5;
            if (Value <= 0)
            {
                //０以下を設定するとエラーになるので0.1を設定する
                Value = 0.1;
            }
            //表示倍率の設定
            cFviImageView1.Display.Magnification = Value;
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// Ascopeチェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ASCOPEの表示を切替えます。
        /// </remarks>
        private void CheckBoxAscope_CheckedChanged(object sender, EventArgs e)
        {
            //ASCOPEオーバレイの設定
            if (CheckBoxAscope.Checked)
            {
                //ASCOPEの表示を有効にする。
                cFviImageView1.Display.OverlayAscope.Enable = true;
                cFviImageView1.Display.OverlayAscope.Active = true;
            }
            else
            {
                //ASCOPEの表示を無効にする。
                cFviImageView1.Display.OverlayAscope.Enable = false;
                cFviImageView1.Display.OverlayAscope.Active = false;
            }
            //再描画
            cFviImageView1.Refresh();

        }

        /// <summary>
        /// ReLoad Ini Fileボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// ビデオ設定用iniファイルの再設定。
        /// </remarks>
        private void ButtonReLoadIni_Click(object sender, EventArgs e)
        {
            try
            {
                //フリーラン中ならば、停止する。
                if (m_ThreadStatus != ThreadStatus.DEAD)
                {
                    fnCaptureStopThread();
                }
                fnCameraSetting();
            }
            catch (Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Save Imageボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 表示画像保存。
        /// </remarks>
        private void ButtonSaveImage_Click(object sender, EventArgs e)
        {

            SaveFileDialog ofdlg = new SaveFileDialog();

            ofdlg.Filter = "画像ファイル(*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat)|*.bmp;*.dib;*.png;*.jpg;*.jpeg;*.tif;*.tiff;*.raw;*.pat";
            ofdlg.Filter += "|Bitmap ファイル (*.bmp;*.dib)|*.bmp;*.dib";
            ofdlg.Filter += "|Png ファイル (*.png)|*.png";
            ofdlg.Filter += "|Jpeg ファイル (*.jpg;*.jpeg)|*.jpg;*.jpeg";
            ofdlg.Filter += "|Tiff ファイル (*.tif;*.tiff)|*.tif;*.tiff";
            ofdlg.Filter += "|Raw ファイル (*.raw)|*.raw";
            ofdlg.Filter += "|すべてのファイル (*.*)|*.*";
            ofdlg.OverwritePrompt = true;

            if (ofdlg.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //この関数は、簡易的に画像ファイルの保存を行う為のものです。 
                    //この関数で指定できないパラメータを指定する必要があれば、 各画像ファイルアクセスクラスを直接使用してください。 
                    FVIL.File.Function.SaveImageFile(ofdlg.FileName, cFviImageView1.Image);

                }
                catch (FVIL.CFviException ex)
                {
                    System.String msg = System.String.Format("ライブラリが例外を発行しました。code={0} func={1}", ex.ErrorCode, ex.Function);
                    switch ((ex.ErrorCode))
                    {
                        case FVIL.ErrorCode.NOT_SUPPORTED:
                        case FVIL.ErrorCode.INVALID_OBJECT:
                            msg += "\r\n" + "サポートされていない画像オブジェクトです。";
                            msg += "\r\n" + "Raw 形式で保存してください。";
                            break;
                    }
                    //エラー処理
                    MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //エラー処理
                    MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void Form_FVC10SAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    if (m_ThreadStatus != ThreadStatus.DEAD)
                    {
                        fnCaptureStopThread();
                    }

                    if (m_grabber.IsGrabbing)
                    {
                        //取り込みの停止
                        m_grabber.GrabAbort();
                        m_grabber.CameraStop();
                    }

                    if (m_grabber.IsAnnounced)
                    {
                        m_grabber.RevokeBuffer();
                    }

                    m_grabber.Close();
                    m_fci.Terminate();
                }
            }
            catch (Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Encoder Modeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダモードの選択。
        /// </remarks>
        private void comboBoxEncoderMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    m_grabber.EncoderMode = comboBoxEncoderMode.SelectedIndex;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Resetsourceコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダリセット設定の選択。
        /// </remarks>
        private void comboBoxResetsource_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    FVIL.FCI.CFviEncoderConfig encoderconfig = new FVIL.FCI.CFviEncoderConfig();
                    encoderconfig = m_grabber.EncoderConfig;
                    encoderconfig.Resetsource = (uint)comboBoxResetsource.SelectedIndex;
                    m_grabber.EncoderConfig = encoderconfig;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Signalsourceコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダー入力チャネルの選択。
        /// </remarks>
        private void comboBoxSignalsource_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    FVIL.FCI.CFviEncoderConfig encoderconfig = new FVIL.FCI.CFviEncoderConfig();
                    encoderconfig = m_grabber.EncoderConfig;
                    encoderconfig.Signalsource = (uint)comboBoxSignalsource.SelectedIndex;
                    m_grabber.EncoderConfig = encoderconfig;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Triggersourceコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// エンコーダ起動トリガの選択。
        /// </remarks>
        private void comboBoxTriggersource_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    FVIL.FCI.CFviEncoderConfig encoderconfig = new FVIL.FCI.CFviEncoderConfig();
                    encoderconfig = m_grabber.EncoderConfig;
                    encoderconfig.Triggersource = (uint)comboBoxTriggersource.SelectedIndex;
                    m_grabber.EncoderConfig = encoderconfig;
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// ScanTypeコンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// カメラタイプの選択。
        /// </remarks>
        private void comboBoxScanType_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                if (m_grabber.IsOpened)
                {
                    if (comboBoxScanType.SelectedIndex == 0)
                    {
                        m_grabber.SetParam("ScanType", "Areascan");
                    }
                    else
                    {
                        m_grabber.SetParam("ScanType", "Linescan");
                    }
                }
            }
            catch (FVIL.CFviException ex)
            {
                //エラー処理
                MessageBox.Show(String.Format("ErrorCode={0}", ex.ErrorCode), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (System.Exception ex)
            {
                //エラー処理
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }

}